@@ -12,6 +12,7 @@ import sys
 import weakref
 import tempfile
 import os
+import imp
 
 from twisted.trial.unittest import TestCase
 from twisted.internet.defer import succeed, Deferred, fail, CancelledError
@@ -485,6 +486,57 @@ EventualResult(Deferred(), None).wait(1.0)
 """)
         self.assertRaises(RuntimeError, __import__, "shouldbeunimportable")
 
+    def test_waiting_during_different_thread_importing(self):
+        """
+        EventualResult.wait() should work if called while a module is
+        being imported in a different thread. See
+        EventualResultTests.test_noWaitingDuringImport for the explanation of
+        what should happen if an import is happening in the current thread.
+        """
+        test_complete = threading.Event()
+        lock_held = threading.Event()
+        er = EventualResult(succeed(123), None)
+
+        def other_thread():
+            imp.acquire_lock()
+            lock_held.set()
+            test_complete.wait()
+            imp.release_lock()
+
+        t = threading.Thread(target=other_thread)
+        t.start()
+        lock_held.wait()
+
+        # While the imp lock is held by the other thread, we can't
+        # allow exceptions/assertions to happen because trial will
+        # try to do an import causing a deadlock instead of a
+        # failure. We collect all assertion pairs (result, expected),
+        # wait for the import lock to be released, and then check our
+        # assertions at the end of the test.
+        assertions = []
+
+        if hasattr(sys, 'pypy_version_info'):
+            # Under PyPy imp.lock_held only returns True if the current thread
+            # holds the lock. If/When that bug is fixed, this assertion
+            # should fail and we should remove this special casing.
+            # See: http://stackoverflow.com/q/23816549/132413
+            assertions.append((imp.lock_held(), False))
+        else:
+            # we want to run .wait while the other thread has the lock acquired
+            assertions.append((imp.lock_held(), True))
+            try:
+                assertions.append((er.wait(), 123))
+            finally:
+                test_complete.set()
+
+            assertions.append((imp.lock_held(), True))
+
+        test_complete.set()
+
+        t.join()
+
+        [self.assertEqual(result, expected) for result, expected in assertions]
+
 
 class InReactorTests(TestCase):
     """
